package jalview.ext.treeviewer;

import jalview.analysis.SequenceIdMatcher;
import jalview.datamodel.SequenceI;
import jalview.util.MappingUtils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class LoadedTreeSequenceAssociation
        implements ExternalLoadedTreeAssociationI
{
  SequenceI[] alignSequences;

  ExternalTreeI tree;

  Map<SequenceI, ExternalTreeNodeI> alignmentWithNodes;

  Map<ExternalTreeNodeI, SequenceI> nodesWithAlignment;

  public LoadedTreeSequenceAssociation(SequenceI[] alignmentSequences,
          ExternalTreeI extTree)
  {
    alignSequences = alignmentSequences;
    tree = extTree;
    alignmentWithNodes = new HashMap<>(alignSequences.length);
    nodesWithAlignment = new HashMap<>(alignSequences.length);

  }

  /**
   * Tries to match sequences from Jalview with tree nodes in Archaeopteryx and
   * fills in the tree node with sequence data if a match is found.
   * 
   * Partially refactored from the old Jalview TreeModel
   * associateLeavesToSequences method.
   *
   * @param seqs
   * @param aptxTree
   */
  @Override
  public void associateLeavesToSequences()

  {
    SequenceIdMatcher algnIds = new SequenceIdMatcher(alignSequences);
    SequenceI nodeSequence;
    String nodeSequenceName;

    if (!tree.isEmpty())
    {
      for (final Iterator<ExternalTreeNodeI> iter = tree
              .iterateInPreOrder(); iter
              .hasNext();)
      {
        ExternalTreeNodeI treeNode = iter.next();
        nodeSequenceName = treeNode.getNodeName();

        nodeSequence = algnIds.findIdMatch(nodeSequenceName);
        if (nodeSequence != null)
        {

          treeNode.setSequence(nodeSequence);

          MappingUtils.putWithDuplicationCheck(alignmentWithNodes,
                  nodeSequence, treeNode);
          MappingUtils.putWithDuplicationCheck(nodesWithAlignment, treeNode,
                  nodeSequence);


        }

    }

    }
  }



  @Override
  public Map<SequenceI, ExternalTreeNodeI> getAlignmentWithNodes()
  {
    return alignmentWithNodes;
  }

  @Override
  public Map<ExternalTreeNodeI, SequenceI> getNodesWithAlignment()
  {
    return nodesWithAlignment;
  }


}






